# <@LICENSE>
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to you under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# </@LICENSE>

=head1 NAME

LRC - local recipient checker plugin

=head1 SYNOPSIS

  loadplugin     Mail::SpamAssassin::Plugin::LRC
  header         LRC_FAILED eval:check_lrc_plugin()

=head1 DESCRIPTION

To try this plugin, write the above two lines in the synopsis to
C</etc/mail/spamassassin/plugintest.cf>.

=cut

package Mail::SpamAssassin::Plugin::LRC;

use Mail::SpamAssassin::Plugin;
use DBI;
use DBD::Pg;
use strict;
use warnings;
use bytes;

use vars qw(@ISA);
@ISA = qw(Mail::SpamAssassin::Plugin);

# constructor: register the eval rule
sub new {
  my $class = shift;
  my $mailsaobject = shift;

  # some boilerplate...
  $class = ref($class) || $class;
  my $self = $class->SUPER::new($mailsaobject);
  bless ($self, $class);

  # the important bit!
  $self->register_eval_rule ("check_lrc_plugin");
  $self->set_config($mailsaobject->{conf});
  print "registered Mail::SpamAssassin::Plugin::LRC: $self\n";
  return $self;
}

sub set_config {
  my($self, $conf) = @_;
    my @cmds = ();

=head1 USER OPTIONS

The following configuration settings are used to control auto-learning:

=over 4

=item lrc_sql_dsn string  (default: dbi:Pg:dbname=database;host=localhost;port=5432)

Connection string to database with list of local mail domains and users

=cut

  push (@cmds, {
      setting => 'lrc_sql_dsn',
      default => "dbi:Pg:dbname=database;host=localhost;port=5432",
      type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING
    });

=item lrc_sql_username string    (default: username)

User name for create connection to database

=cut

  push (@cmds, {
      setting => 'lrc_sql_username',
      default => "username",
      type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING
    });

=item lrc_sql_password string    (default: password)

User password for create connection to database

=cut
  push (@cmds, {
      setting => 'lrc_sql_password',
      default => "password",
      type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING
    });

=item lrc_sql_domains_table string    (default: mailertable)

Where is stored local domains list?

=cut
  push (@cmds, {
      setting => 'lrc_sql_domains_table',
      default => "mailertable",
      type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING
    });

=item lrc_sql_domains_field string    (default: s_in)

Where is stored local domains list?

=cut
  push (@cmds, {
      setting => 'lrc_sql_domains_field',
      default => "s_in",
      type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING
    });

=item lrc_sql_users_table string    (default: users)

Where is stored local users list?

=cut
  push (@cmds, {
      setting => 'lrc_sql_users_table',
      default => "users",
      type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING
    });

=item lrc_sql_users_field string    (default: login)

Where is stored local users list?

=cut
  push (@cmds, {
      setting => 'lrc_sql_users_field',
      default => "login",
      type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING
    });

=item lrc_sql_aliases_table string    (default: aliases_in)
Where is stored local aliases list?

=cut
  push (@cmds, {
      setting => 'lrc_sql_aliases_table',
      default => "aliases_in",
      type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING
    });

=item lrc_sql_aliases_field string    (default: alias)

Where is stored local aliases list?

=cut
  push (@cmds, {
      setting => 'lrc_sql_aliases_field',
      default => "alias",
      type => $Mail::SpamAssassin::Conf::CONF_TYPE_STRING
    });


 $conf->{parser}->register_commands(\@cmds);
 }


sub check_domain
  {
  my ($dbh, $sql_domains_table, $sql_domains_field, $domainname) = @_;
  my $rv;
  my $result;
  $rv=$dbh->prepare("SELECT $sql_domains_field FROM $sql_domains_table WHERE $sql_domains_field=lower('$domainname')");
  $rv->execute;
  $result = $rv->rows;
  $rv->finish() if($rv);
  if ($result > 0) {return 1;}
  return 0;
  }

# and the eval rule itself
sub check_lrc_plugin {
  my ($self, $permsgstatus) = @_;
  print "Mail::SpamAssassin::Plugin::LRC eval test called: $self\n";
  # ... hard work goes here...
  my $db_user  = $self->{main}->{conf}->{lrc_sql_username};
  my $db_pass  = $self->{main}->{conf}->{lrc_sql_password};
  my $sql_dsn  = $self->{main}->{conf}->{lrc_sql_dsn};
  my $sql_domains_table  = $self->{main}->{conf}->{lrc_sql_domains_table};
  my $sql_domains_field  = $self->{main}->{conf}->{lrc_sql_domains_field};
  my $sql_users_table  = $self->{main}->{conf}->{lrc_sql_users_table};
  my $sql_users_field  = $self->{main}->{conf}->{lrc_sql_users_field};
  my $sql_aliases_table  = $self->{main}->{conf}->{lrc_sql_aliases_table};
  my $sql_aliases_field  = $self->{main}->{conf}->{lrc_sql_aliases_field};
  my $dbh;
  my $rv;
  my $score_in_config = $self->{main}->{conf}->{scores}->{'LRC_FAILED'};
  my $score = 0;
  my $failed_mail_addresses = "";

    $dbh = DBI->connect($sql_dsn, "$db_user", "$db_pass");
    if(!$dbh)
      {
      print "Mail::SpamAssassin::Plugin::LRC error:" + $DBI::errstr + "\n";
      die "Oops!!!";
      }

    print "Mail::SpamAssassin::Plugin::LRC: connect to db\n";

    foreach my $mail_address ($permsgstatus->all_from_addrs())
      {
  # $user."\@"
  # rotate through $domain and check
      my ($user,$domain) = split(/\@/, $mail_address,2);
      if (check_domain($dbh, $sql_domains_table, $sql_domains_field, $domain))
        {
        print "Mail::SpamAssassin::Plugin::LRC: $domain is local\n";
        $rv=$dbh->prepare("SELECT $sql_users_field FROM $sql_users_table WHERE  $sql_users_field=lower('$mail_address')");
        $rv->execute;
        if ($rv->rows < 1)
          {
          print "Mail::SpamAssassin::Plugin::LRC: MAIL FROM: $mail_address not exist\n";
          $rv->finish() if($rv);
          $permsgstatus->_handle_hit("LRC_FAILED", $score_in_config * 10, "HEADER: ", "MAIL FROM: $mail_address not exist");
          $dbh->disconnect if ($dbh);
          return 0;
          }
        else
          {
          $rv->finish() if($rv);
          print "Mail::SpamAssassin::Plugin::LRC: MAIL FROM: $mail_address is exist\n";
          }
        }
      else
        {
        print "Mail::SpamAssassin::Plugin::LRC: $domain is alien\n";
        }
      }

    foreach my $mail_address ($permsgstatus->all_to_addrs())
      {
  # $user."\@"
  # rotate through $domain and check
      my ($user,$domain) = split(/\@/, $mail_address,2);
      if (check_domain($dbh, $sql_domains_table, $sql_domains_field, $domain))
        {
        print "Mail::SpamAssassin::Plugin::LRC: $domain is local\n";
        my $search_status = 0;
        $rv=$dbh->prepare("SELECT $sql_users_field FROM $sql_users_table WHERE $sql_users_field=lower('$mail_address')");
        $rv->execute;
        if ($rv->rows >= 1) {$search_status = 1}
        $rv->finish() if($rv);

        $rv=$dbh->prepare("SELECT $sql_aliases_field FROM $sql_aliases_table WHERE $sql_aliases_field=lower('$mail_address')");
        $rv->execute;
        if ($rv->rows >= 1) {$search_status = 1}
        $rv->finish() if($rv);

        if ($search_status < 1)
          {
          print "Mail::SpamAssassin::Plugin::LRC: RCPT TO: $mail_address not exist\n";
          $score++;
          $failed_mail_addresses = "$failed_mail_addresses, $mail_address";
          }
        else
          {
          print "Mail::SpamAssassin::Plugin::LRC: RCPT TO: $mail_address is exist\n";
          }
        }
      else
        {
        print "Mail::SpamAssassin::Plugin::LRC: $domain is alien\n";
        }
      }
  $dbh->disconnect if ($dbh);
  if ($score > 0)
    {
    $permsgstatus->_handle_hit("LRC_FAILED", $score * $score_in_config, "HEADER: ", "RCPT TO: $failed_mail_addresses not exist");
    }
  return 0;
}

1;

=back

=cut
